###############################################################
### custom functions
###############################################################

dnl NAW_REQ_PATH(libname, error-if-non-existent)
dnl
AC_DEFUN(NAW_REQ_PATH, [
	AC_MSG_CHECKING(which directory to use for $1)
	if test -z "$ac_path_$1" -o "$ac_path_$1" = "no" ; then
	  AC_MSG_RESULT(unknown)
	  AC_MSG_ERROR($2)
	else
	  AC_MSG_RESULT($ac_path_$1)
	fi

	LDFLAGS="-L$ac_path_$1 $LDFLAGS"
	CFLAGS="-I$ac_path_$1 $CFLAGS"
])

dnl NAW_USE_PKG(pkgname, dfltpath, helpstring, if-yes, if-no)
dnl
AC_DEFUN(NAW_USE_PKG, [
	AC_ARG_WITH($1,
		[AC_HELP_STRING([--with-$1=path], [$3])],
		[naw_pkg_path_$1=$withval],
		[naw_pkg_path_$1=$2])

        test "$naw_pkg_path_$1" = "yes" && naw_pkg_path_$1=

	AC_MSG_CHECKING(directory for $1)

        if test "$naw_pkg_path_$1" = "no" ; then
          AC_MSG_RESULT(no)
	  $5
        else
	  if test -z "$naw_pkg_path_$1"; then
	    AC_MSG_RESULT(not given)
          else
	    AC_MSG_RESULT($naw_pkg_path_$1)

	    # use full path
	    naw_pkg_path_$1=`cd $naw_pkg_path_$1 && pwd`
	    if test $? -ne 0; then
	      AC_MSG_ERROR(error gathering full path)
	    fi

	    test ! -z "$naw_use_LDFLAGS_$1" && \
	      naw_pkg_LDFLAGS_$1="-L$naw_pkg_path_$1/$naw_use_LDFLAGS_$1"
	    test ! -z "$naw_use_CFLAGS_$1" && \
	      naw_pkg_CFLAGS_$1="-I$naw_pkg_path_$1/$naw_use_CFLAGS_$1"
	    test ! -z "$naw_use_LIBS_$1" && \
	      for lib in $naw_use_LIBS_$1; do \
	        naw_pkg_LIBS_$1="$naw_pkg_LIBS_$1 $naw_pkg_path_$1/$lib"; \
	      done
          fi

	  $4
        fi
])

dnl NAW_USE_PKG_REQ(pkgname, helpstring, error-if-no)
dnl
AC_DEFUN(NAW_USE_PKG_REQ, [
	NAW_USE_PKG([$1],[$2],[$3],,[AC_MSG_ERROR([$4])])
])

AC_DEFUN(NAW_PKG_FLAGS_INIT, [
	saved_CPPFLAGS=$CPPFLAGS
	saved_LDFLAGS=$LDFLAGS
	saved_CFLAGS=$CFLAGS
	saved_LIBS=$LIBS

	LDFLAGS="$naw_pkg_LDFLAGS_$1 $LDFLAGS"
	CFLAGS="$naw_pkg_CFLAGS_$1 $CFLAGS"
	LIBS="$naw_pkg_LIBS_$1 $LIBS"
	CPPFLAGS="$CFLAGS"
])

AC_DEFUN(NAW_PKG_FLAGS_REVERT, [
	CPPFLAGS=$saved_CPPFLAGS
	LDFLAGS=$saved_LDFLAGS
	CFLAGS=$saved_CFLAGS
	LIBS=$saved_LIBS
])

AC_DEFUN(NAW_PKG_FLAGS_SET_SUBST, [
	AC_SUBST($1_LDFLAGS, $naw_pkg_LDFLAGS_$1)
	AC_SUBST($1_CFLAGS, $naw_pkg_CFLAGS_$1)
	AC_SUBST($1_LIBS, $naw_pkg_LIBS_$1)
])

dnl NAW_PKG_FLAGS_COPY(sourcepkg, destpkg)
AC_DEFUN(NAW_PKG_FLAGS_COPY, [
	naw_pkg_LIBS_$2=$naw_pkg_LIBS_$1
	naw_pkg_CFLAGS_$2=$naw_pkg_CFLAGS_$1
	naw_pkg_LDFLAGS_$2=$naw_pkg_LDFLAGS_$1
	NAW_PKG_FLAGS_SET_SUBST($1)
])

dnl NAW_CHECK_LIB(pkgname, ... args to AC_CHECK_LIB ...)
dnl
AC_DEFUN(NAW_CHECK_LIB, [
	NAW_PKG_FLAGS_INIT($1)
	if test -z "$4" -a "$2" != "c"; then
	  AC_CHECK_LIB($2,$3,naw_pkg_LIBS_$1="-l$2",$5,$6)
	else
	  AC_CHECK_LIB($2,$3,$4,$5,$6)
	fi

	NAW_PKG_FLAGS_SET_SUBST($1)
	NAW_PKG_FLAGS_REVERT
])

dnl NAW_CHECK_HEADERS(headernames, flagnames, act-if-found, act-if-not-found)
AC_DEFUN(NAW_CHECK_HEADERS, [
        NAW_PKG_FLAGS_INIT($1)
        AC_CHECK_HEADERS($2,$3,$4)
        NAW_PKG_FLAGS_REVERT
])

dnl NAW_CHECK_OBJS(pkgname,function,action-if-found,action-if-not-found,objectfiles)
dnl
AC_DEFUN(NAW_CHECK_OBJS, [
	NAW_PKG_FLAGS_INIT($1)
	AC_MSG_CHECKING(for $2 in $1 objects)
	NAW_PKG_FLAGS_REVERT
])

dnl check special flags related to $1
dnl evaluates value of NAW_BUILD_$1
dnl sets substitution value to the same
AC_DEFUN(NAW_CHECK_SPECIAL,
	[
	  AC_MSG_CHECKING([whether you need special $1])
	  if test ! -z "$NAW_BUILD_$1"; then
	    AC_MSG_RESULT([[$NAW_BUILD_$1]])
	  else
	    AC_MSG_RESULT(no)
	  fi
	  AC_SUBST([NAW_BUILD_$1])
	]
)

AC_DEFUN(NAW_CONFIG_FILES, [
	AC_SUBST(CONFIGFILES, "$CONFIGFILES $1")
	AC_CONFIG_FILES($1)
])

#AC_DEFUN(NAW_CHECK_DLFCN, [
#	AC_MSG_CHECKING([whether your system supports RTLD_NEXT])
#	AC_COMPILE_IFELSE(
#		[#include <dlfcn.h>
#		char *dltest = RTLD_NEXT;],
#		[AC_MSG_RESULT(yes)]
#		[AC_DEFINE(HAVE_RTLD_NEXT, 1,
#		  [whether you have RTLD_NEXT defined in dlfcn.h])],
#		[AC_MSG_RESULT(no)])
#])

