#include "auto-stage.h"

#include <sys/types.h>
#include <string.h>
#include <errno.h>

/* get around sys/socket.h accept() declaration */
#ifdef NAW_PRELOAD
#define accept inc_accept_proto
#endif
# include <sys/socket.h>
#undef accept

#include "lxmisc.h"
#include "naw_log.h"
#include "misc.h"

#ifdef NAW_PRELOAD
static int (*realaccept)(int, struct sockaddr *, socklen_t *);

void wrap_accept (void) {
    realaccept = get_funcptr ("accept");
}
#else
# define realaccept accept
#endif

int naw_accept (sockfd, addr, addrlen)
    int sockfd;
    struct sockaddr *addr;
    socklen_t *addrlen;
{
    int r;

    EXPLICIT_INIT(if (!naw_initialized) naw_init());

    NAW_LOG (NAW_DEBUG2, LXM_q("accept() was called."));

    /* optionally act as blocking */
    for (;;) {
        r = realaccept (sockfd, addr, addrlen);
        if (r == -1) {
            naw_logfmt(NAW_DEBUG, "accept failed: %s", strerror(errno));
            return (r); /* covers nonblocking sockets */
        }

        /* judge the address; return with sentence */
        /* imposed and errno set                   */
        r = naw_verify_internal (r, INIT_SRV, addr);
        if (r == -1) NAW_LOG (NAW_NOTICE, LXM_q("accept() denied"));
        if (!force_block || r >= 0) return r;
        NAW_LOG (NAW_DEBUG2, LXM_q("accept(): trying again..."));
    }
}
